//============================================================================
// Name        : load.cpp
// Author      : 
// Version     :
// Copyright   : Your copyright notice
// Description : Ansi-style
//============================================================================
 
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <iostream>
#include <string>
#include <cmath>
#include <exception>
using namespace std;

struct MatMulException : public exception {
   const char * what () const throw () {
       return "Unable to multiply matrices due to incorrect dimensions";
   }
};

class AccessException : public exception
{
    const char *what() const throw(){
        return "Attempted to access an index that is out of range";
    }
};

struct MatrixElementWiseException : public exception
{
    const char *what() const throw()
    {
        return "Unable to perform element-wise operation between matrices of different dimensions";
    }
};

typedef struct
{
    double *arr;
    int32_t r;
    int32_t c;
} matrix_t;

extern "C" int sprintm(double *indata, int r, int c)
{
    putchar('[');
    for (int i = 0; i < r; ++i){
        for (int j = 0; j < c; ++j){
            if (i > 0 && j == 0){
                putchar(' ');
            }
            printf("%.1f", indata[i * c + j]);
            if (j < c - 1){
                putchar('\t');
            }
        }
        if(i < r-1){
            putchar('\n');
        }
    }
    putchar(']');
    putchar('\n');

    return 0;
}

extern "C" void strans(double *indata, double *outdata, int r, int c)
{
    for (int i = 0; i < r; ++i)
    {
        for (int j = 0; j < c; ++j)
        {
            outdata[j * r + i] = indata[i * c + j];
        }
    }
    return;
}

extern "C" bool check_range(int r, int c, int x, int y)
{
    if (x >= 0 && x < r && y >= 0 && y < c)
    {
        return true;
    }
    else
    {
        return false;
    }
}

extern "C" int saccess(double *indata, int r, int c, int i, int j)
{
    int idx;
    if (check_range(r, c, i, j))
    {
        idx = (i * c) + j;
    }
    else{
        throw AccessException();
    }
    return idx;
}

extern "C" double ssum(double *indata, int r, int c)
{
    double total = 0.0;
    for (int i = 0; i < r*c; ++i)
    {
        total += indata[i];
    
    }
    return total;
}

extern "C" void sconsmul(double *indata, double *outdata, int r, int c, double cc)
{

    for (int i = 0; i < r * c; ++i)
    {
        outdata[i] = cc * indata[i];
    }
}

extern "C" void sconsdiv(double *indata, double *outdata, int r, int c, double cc)
{

    for (int i = 0; i < r * c; ++i)
    {
        outdata[i] = indata[i] / cc;
    }
}

extern "C" void smatmul(double *indata1, double *indata2, double *outdata, int r1, int c1, int r2, int c2)
{

    if (c1 != r2) {
        throw MatMulException();
    }
    else {

        for (int i = 0; i < r1; ++i)
        {
            for (int j = 0; j < c2; ++j)
            {
                int idx = i * c2 + j;
                float tmp_sum = 0.0;
                int x = i * c1;
                int y = j;
                while (y < r2 * c2)
                {
                    tmp_sum += indata1[x] * indata2[y];
                    x = x + 1;
                    y = y + c2;
                }
                outdata[idx] = tmp_sum;
            }
        }
        return;
    }
}

extern "C" void smatslice(matrix_t *inmat, int r, int c, int i, int j, int k, int l, matrix_t *outmat)
{
    if (check_range(r, c, i, k) && check_range(r, c, j, l))
    {
        int outr = outmat->r = j - i + 1, outc = outmat->c = l - k + 1;
        double *indata = inmat->arr,
            *outdata = outmat->arr = (double *)malloc((outr * outc) * sizeof(double));
        int x, y;

        assert(outdata != NULL);

        for (x = i; x <= j; ++x)
            for (y = k; y <= l; ++y)
                outdata[(x - i) * outc + (y - k)] = indata[x * inmat->c + y];
    }
    else{
        throw AccessException();
    }
}

extern "C" int smateleassign(double *indata1, int r, int c, int i, int j, float ele)
{
    int idx;
    indata1[i * c + j] = ele;
    if (check_range(r, c, i, j))
    {
        idx = (i * c) + j;
        indata1[i * c + j] = ele;
    }
    else
    {
        throw AccessException();
    }
    return idx;
}

extern "C" void selemul(double *indata1, double *indata2, double *outdata, int r1, int c1, int r2, int c2)
{
    if (r1 == r2 and c1 == c2){
        for (int i = 0; i < r1*c1; ++i)
        {
            outdata[i] = indata1[i] * indata2[i];
        }
    }
    else
    {
        throw MatrixElementWiseException();
    }
}

extern "C" void selediv(double *indata1, double *indata2, double *outdata, int r1, int c1, int r2, int c2)
{

    if (r1 == r2 and c1 == c2)
    {
        for (int i = 0; i < r1 * c1; ++i)
        {
            outdata[i] = indata1[i] / indata2[i];
        }
    }
    else
    {
        throw MatrixElementWiseException();
    }
}

extern "C" double smean(double *indata, int r, int c)
{
    return ssum(indata, r, c) / (r * c);
}
