#include <opencv2/core.hpp>
#include <opencv2/imgproc.hpp>
#include <opencv2/imgcodecs.hpp>

using namespace cv;

typedef struct {
	double *arr;
	int32_t r;
	int32_t c;
} matrix_t;

#define MAT(mat) \
	Mat(mat->r, mat->c, CV_64F, mat->arr, sizeof(double) * mat->c)

static inline matrix_t YMAT_1B(Mat mat)
{
	double *arr = (double *)malloc((mat.rows * mat.cols) * sizeof(double));
	matrix_t ymat = { .arr = arr, .r = mat.rows, .c = mat.cols };
	int i;

	assert(ymat.arr != NULL);
	for (i = 0; i < ymat.r * ymat.c; ++i)
		ymat.arr[i] = (double)((unsigned char *)mat.data)[i];

	return ymat;
}

extern "C" void sfilter2D(matrix_t *src, matrix_t *dst, int ddepth, matrix_t *kernel)
{
	filter2D(MAT(src), MAT(dst), ddepth, MAT(kernel));
}

extern "C" void sinvert(matrix_t *src, matrix_t *dst)
{
	invert(MAT(src),MAT(dst),DECOMP_LU);
}

extern "C" void simread(const char *filename, matrix_t *dst)
{
	Mat mat = imread(filename, IMREAD_GRAYSCALE);
	*dst = YMAT_1B(mat);
}

extern "C" bool simwrite(const char *filename, matrix_t *img)
{
	return imwrite(filename, MAT(img));
}
 
